@@ -90,6 +90,8 @@ class PreprocessInputRetriever():
         environ_base = response.pop('environ_base', None)
         if 'files' in kwargs:
             request = wrappers.Request.from_values(
+                data = data,
+                json = json,
                 input_stream = BytesIO(files_data),
                 content_length = len(files_data),
                 content_type = "multipart/form-data; boundary=boundary",
@@ -102,21 +104,23 @@ class PreprocessInputRetriever():
             for key, value in response.items():
                 setattr(request, key, value)
         elif environ_base:
-            request = wrappers.Request.from_values(environ_base = environ_base)
+            request = wrappers.Request.from_values(
+                data = data,
+                json = json,
+                environ_base = environ_base
+            )
             for key, value in response.items():
                 setattr(request, key, value)
         else:
             parsed_result = urllib.parse.urlparse(url)
             localhost_url = url.replace(parsed_result.netloc, 'localhost')
-            request = wrappers.Request.from_values(localhost_url)
+            request = wrappers.Request.from_values(
+                localhost_url,
+                data = data,
+                json = json
+            )
             for key, value in response.items():
                 setattr(request, key, value)
-
-        if data:
-            request.data = data
-        elif json:
-            request.data = dumps(json)
-
         return request
 
     @staticmethod
@@ -139,18 +143,21 @@ class PreprocessInputRetriever():
         response = response.json()
         environ_base = response.pop('environ_base', None)
         if environ_base:
-            request = wrappers.Request.from_values(environ_base = environ_base)
+            request = wrappers.Request.from_values(
+                data = data,
+                environ_base = environ_base
+            )
         else:
             parsed_result = urllib.parse.urlparse(url)
             localhost_url = url.replace(parsed_result.netloc, 'localhost')
-            request = wrappers.Request.from_values(localhost_url)
+            request = wrappers.Request.from_values(
+                localhost_url,
+                data = data
+            )
 
         for key, value in response.items():
             setattr(request, key, value)
 
-        if data:
-            request.data = data
-
         if 'files' in kwargs:
             files = kwargs.get('files')
             request.files = files
